==> coventry-0.8.1.tar.gz: coventry-0.8.1/utils/pbx-presence.cpp <==
/*
 * Copyright (C) 2022 Tycho Softworks.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <moderncli/print.hpp>
#include <moderncli/strings.hpp>
#include <moderncli/args.hpp>

#include <unistd.h>
#include <fcntl.h>
#include <mqueue.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <arpa/inet.h>
#include <netdb.h>

#include "server.hpp"
#include "ipc.hpp"

namespace {
args::option helpflag('h',  "--help",       "display this list");
args::option quiet('q',     "--quiet",      "suppress all output");
args::option althelp('?',   nullptr, nullptr);

class ipc_error final : public std::exception {
public:
    explicit ipc_error(const char *text, int code) noexcept : msg(text), err(code) {}

    auto what() const noexcept -> const char * override {
        return msg;
    }

    auto exit() const noexcept {
        return err;
    }

private:
    const char *msg;
    int err;
};

auto get_value(std::string_view text) -> int {
    int value = 0;
    bool negative = false;

    if(!text.empty() && text.front() == '-') {
        negative = true;
        text.remove_prefix(1);
    }

    for(auto ch : text) {
        if(ch < '0' || ch > '9')
            return 0;
        if(value > 99999999)
            return 0;
        value *= 10;
        value += (ch - '0');
    }

    if(negative)
        return -value;

    return value;
}

auto set_string(char *cp, size_t max, const char *from) -> size_t {
    assert(cp != nullptr);
    assert(from != nullptr);
    assert(max > 0);

    auto count = 0UL;
    while(--max && *from) {
        *(cp++) = *(from++);
        ++count;
    }
    *cp = 0;
    return count;
}
} // end namespace

auto main(int argc, const char **argv) -> int { // NOLINT
    try {
        auto count = args::parse(argc, argv);
        auto list = args::list();
        auto extension = 0;

        if(is(helpflag) || is(althelp) || count < 1 || count > 2) {
            args::help({"pbx-presence extension [presence]"}, {
                "pbx-presence " PROJECT_VERSION,
                "David Sugar <tychosoft@gmail.com",
                "Examine or set pbx presence for an extension"
            });
            return 0;
        }

        extension = get_value(list[0]);
        if(count == 2) {
            auto presence = lower_case(std::string{list[1]});
            pbx_msg msg{.type = PBX_PRESENCE, .ver = PBX_VERSION};
            msg.body.registry.ext = extension;
            set_string(msg.body.registry.value, sizeof(msg.body.registry.value) - 1, presence.c_str());

            auto mq = mq_open(SERVER_IPCPATH, O_WRONLY, 0660, nullptr);
            if(mq == MQD_INVALID)
                throw ipc_error("mqueue failed", 4);
            if(mq_send(mq, reinterpret_cast<const char *>(&msg), sizeof(msg), 0) < 0)
                throw ipc_error("send failed", 4);
        }
        else {  // count == 1
            auto rshm = shm_open(SERVER_IPCPATH ".registry", O_RDONLY, 0660);
            if(rshm < 0)
                throw ipc_error("failed", 4);
            struct stat rino{};
            if(fstat(rshm, &rino) < 0)
                throw ipc_error("failed", 4);
            auto rmax = (rino.st_size - sizeof(pbx_sys)) / sizeof(pbx_reg);
            auto rmem = mmap(nullptr, sizeof(pbx_reg) * rmax + sizeof(pbx_sys), PROT_READ, MAP_SHARED, rshm, 0);
            auto reg = static_cast<pbx_reg*>(rmem);
            auto sys = reinterpret_cast<const pbx_sys *>(&reg[rmax]);
            if(extension < sys->first || extension >= static_cast<int>(sys->first + rmax))
                throw ipc_error("offline", 1);

            const auto& user = reg[extension - sys->first];
            auto presence = "here";
            switch(user.presence) {
            case pbx_reg::HERE:
                if(user.count >= user.lines)
                    presence = "busy";
                else if(user.count)
                    presence = "call";
                else
                    presence = "here";
                break;
            case pbx_reg::AWAY:
                presence = "away";
                break;
            case pbx_reg::DND:
                presence = "dnd";
                break;
            default:
                break;
            }
            print("{}\n", presence);
        }
    }
    catch(const bad_arg& e) {
        if(is(quiet))
            ::exit(-1);
        die(-1, "Usage: pbx-presence extensions [presence]\nerror: {}\n", e.what());
    }
    catch(const ipc_error& e) {
        if(is(quiet))
            ::exit(e.exit());
        die(e.exit(), "error: {}\n", e.what());
    }
    catch(const std::exception& e) {
        if(is(quiet))
            ::exit(-1);
        die(-1, "error: {}\n", e.what());
    }
    catch(...) {
        if(is(quiet))
            ::exit(-1);
        die(-1, "error: unknown error\n");
    }
}

==> coventry-0.8.6.tar.gz: coventry-0.8.6/utils/pbx-presence.cpp <==
/*
 * Copyright (C) 2022 Tycho Softworks.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "server.hpp"       // IWYU pragma: keep
#include "ipc.hpp"

#include <moderncli/strings.hpp>
#include <moderncli/process.hpp>
#include <moderncli/args.hpp>

#include <mqueue.h>
#include <sys/stat.h>

namespace {
args::option helpflag('h',  "--help",       "display this list");
args::option quiet('q',     "--quiet",      "suppress all output");
args::option althelp('?',   nullptr, nullptr);

class ipc_error final : public std::exception {
public:
    explicit ipc_error(const char *text, int code) noexcept : msg(text), err(code) {}

    auto what() const noexcept -> const char * override {
        return msg;
    }

    auto exit() const noexcept {
        return err;
    }

private:
    const char *msg;
    int err;
};

auto get_value(std::string_view text) -> int {
    int value = 0;
    bool negative = false;

    if(!text.empty() && text.front() == '-') {
        negative = true;
        text.remove_prefix(1);
    }

    for(auto ch : text) {
        if(ch < '0' || ch > '9')
            return 0;
        if(value > 99999999)
            return 0;
        value *= 10;
        value += (ch - '0');
    }

    if(negative)
        return -value;

    return value;
}

auto set_string(char *cp, size_t max, const char *from) -> size_t {
    assert(cp != nullptr);
    assert(from != nullptr);
    assert(max > 0);

    auto count = 0UL;
    while(--max && *from) {
        *(cp++) = *(from++);
        ++count;
    }
    *cp = 0;
    return count;
}
} // end namespace

auto main(int argc, const char **argv) -> int {
    using namespace process;

    try {
        auto count = args::parse(argc, argv);
        auto list = args::list();
        auto extension = 0;

        if(is(helpflag) || is(althelp) || count < 1 || count > 2) {
            args::help({"pbx-presence extension [presence]"}, {
                "pbx-presence " PROJECT_VERSION,
                "David Sugar <tychosoft@gmail.com",
                "Examine or set pbx presence for an extension"
            });
            return 0;
        }

        extension = get_value(list[0]);
        if(count == 2) {
            auto presence = lower_case(std::string{list[1]});
            pbx_msg msg{.type = PBX_PRESENCE, .ver = PBX_VERSION};
            msg.body.registry.ext = extension;
            set_string(msg.body.registry.value, sizeof(msg.body.registry.value) - 1, presence.c_str());

            auto mq = mq_open(SERVER_IPCPATH, O_WRONLY, 0660, nullptr);
            if(mq == MQD_INVALID)
                throw ipc_error("mqueue failed", 4);
            if(mq_send(mq, reinterpret_cast<const char *>(&msg), sizeof(msg), 0) < 0)
                throw ipc_error("send failed", 4);
        }
        else {  // count == 1
            auto rshm = shm_open(SERVER_IPCPATH ".registry", O_RDONLY, 0660);
            if(rshm < 0)
                throw ipc_error("failed", 4);
            struct stat rino{};
            if(fstat(rshm, &rino) < 0)
                throw ipc_error("failed", 4);
            auto rmax = (rino.st_size - sizeof(pbx_sys)) / sizeof(pbx_reg);
            auto rmem = map_t(rshm, rino.st_size, false);
            if(!is(rmem))
                throw ipc_error("failed", 4);
            auto reg = static_cast<pbx_reg*>(rmem.data());
            auto sys = reinterpret_cast<const pbx_sys *>(&reg[rmax]);
            if(extension < sys->first || extension >= static_cast<int>(sys->first + rmax))
                throw ipc_error("offline", 1);

            const auto& user = reg[extension - sys->first];
            auto presence = "here";
            switch(user.presence) {
            case pbx_reg::HERE:
                if(user.count >= user.lines)
                    presence = "busy";
                else if(user.count)
                    presence = "call";
                else
                    presence = "here";
                break;
            case pbx_reg::AWAY:
                presence = "away";
                break;
            case pbx_reg::DND:
                presence = "dnd";
                break;
            default:
                break;
            }
            print("{}\n", presence);
        }
    }
    catch(const bad_arg& e) {
        if(is(quiet))
            ::exit(-1);
        die(-1, "Usage: pbx-presence extensions [presence]\nerror: {}\n", e.what());
    }
    catch(const ipc_error& e) {
        if(is(quiet))
            ::exit(e.exit());
        die(e.exit(), "error: {}\n", e.what());
    }
    catch(const std::exception& e) {
        if(is(quiet))
            ::exit(-1);
        die(-1, "error: {}\n", e.what());
    }
    catch(...) {
        if(is(quiet))
            ::exit(-1);
        die(-1, "error: unknown error\n");
    }
}
